@@ -7,15 +7,12 @@ See https://github.com/kislyuk/yq for more information.
 
 # PYTHON_ARGCOMPLETE_OK
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import sys, argparse, subprocess, json
+import os, sys, argparse, subprocess, json
 from collections import OrderedDict
 from datetime import datetime, date, time
 
 import yaml, argcomplete
 
-from .compat import USING_PYTHON2, open
 from .parser import get_parser, jq_arg_spec
 from .loader import get_loader
 from .dumper import get_dumper
@@ -37,7 +34,7 @@ def xq_cli():
     cli(input_format="xml", program_name="xq")
 
 def tq_cli():
-    cli(input_format="toml", program_name="tq")
+    cli(input_format="toml", program_name="tomlq")
 
 class DeferredOutputStream:
     def __init__(self, name, mode="w"):
@@ -66,9 +63,12 @@ def cli(args=None, input_format="yaml", program_name="yq"):
     parser = get_parser(program_name, __doc__)
     argcomplete.autocomplete(parser)
     args, jq_args = parser.parse_known_args(args=args)
+    null_input = False
 
     for i, arg in enumerate(jq_args):
         if arg.startswith("-") and not arg.startswith("--"):
+            if "n" in arg:
+                null_input = True
             if "i" in arg:
                 args.in_place = True
             if "y" in arg:
@@ -92,7 +92,6 @@ def cli(args=None, input_format="yaml", program_name="yq"):
             for value_group in values:
                 jq_args.append(arg)
                 jq_args.extend(value_group)
-
     if args.jq_filter is not None:
         if "--from-file" in jq_args or "-f" in jq_args:
             args.input_streams.insert(0, argparse.FileType()(args.jq_filter))
@@ -103,6 +102,8 @@ def cli(args=None, input_format="yaml", program_name="yq"):
             elif "--jsonargs" in jq_args:
                 jq_filter_arg_loc = jq_args.index('--jsonargs') + 1
             jq_args.insert(jq_filter_arg_loc, args.jq_filter)
+            if null_input:
+                args.input_streams.insert(0, open(os.devnull))
     delattr(args, "jq_filter")
     in_place = args.in_place
     delattr(args, "in_place")
@@ -115,8 +116,6 @@ def cli(args=None, input_format="yaml", program_name="yq"):
 
     yq_args = dict(input_format=input_format, program_name=program_name, jq_args=jq_args, **vars(args))
     if in_place:
-        if USING_PYTHON2:
-            sys.exit("{}: -i/--in-place is not compatible with Python 2".format(program_name))
         if args.output_format not in {"yaml", "annotated_yaml"}:
             sys.exit("{}: -i/--in-place can only be used with -y/-Y".format(program_name))
         input_streams = yq_args.pop("input_streams")
@@ -145,10 +144,12 @@ def yq(input_streams=None, output_stream=None, input_format="yaml", output_forma
     converting_output = True if output_format != "json" else False
 
     try:
-        # Note: universal_newlines is just a way to induce subprocess to make stdin a text buffer and encode it for us
+        # Notes: universal_newlines is just a way to induce subprocess to make stdin a text buffer and encode it for us;
+        # close_fds must be false for command substitution to work (yq . t.yml --slurpfile t <(yq . t.yml))
         jq = subprocess.Popen(["jq"] + list(jq_args),
                               stdin=subprocess.PIPE,
                               stdout=subprocess.PIPE if converting_output else None,
+                              close_fds=False,
                               universal_newlines=True)
     except OSError as e:
         msg = "{}: Error starting jq: {}: {}. Is jq installed and available on PATH?"
@@ -207,14 +208,7 @@ def yq(input_streams=None, output_stream=None, input_format="yaml", output_forma
                     if not isinstance(doc, OrderedDict):
                         msg = "{}: Error converting JSON to TOML: cannot represent non-object types at top level."
                         exit_func(msg.format(program_name))
-
-                    if USING_PYTHON2:
-                        # For Python 2, dump the string and encode it into bytes.
-                        output = toml.dumps(doc)
-                        output_stream.write(output.encode("utf-8"))
-                    else:
-                        # For Python 3, write the unicode to the buffer directly.
-                        toml.dump(doc, output_stream)
+                    toml.dump(doc, output_stream)
         else:
             if input_format == "yaml":
                 loader = get_loader(use_annotations=False)
